@@ -10,7 +10,7 @@
 #	pragma hdrstop
 #endif
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	define ASCII_TRANSLATION 0
 #endif
 
@@ -46,10 +46,10 @@ FTPGetFiles3(
 	const FTPConfirmResumeDownloadProc resumeProc,
 	int UNUSED(reserved))
 {
-	LineList globList;
-	LinePtr itemPtr;
-	FileInfoList files;
-	FileInfoPtr filePtr;
+	FTPLineList globList;
+	FTPLinePtr itemPtr;
+	FTPFileInfoList files;
+	FTPFileInfoPtr filePtr;
 	int batchResult;
 	int result;
 	char *ldir;
@@ -160,7 +160,7 @@ FTPGetFiles3(
 				break;
 			}
 			if (filePtr->type == 'd') {
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 				(void) MkDirs(filePtr->lname, 00777);
 #else
 				(void) mkdir(filePtr->lname, 00777);
@@ -195,7 +195,7 @@ FTPGetFiles3(
 						c = *cp;
 						*cp = '\0';
 						if (MkDirs(ldir, 00777) < 0) {
-							Error(cip, kDoPerror, "Could not create local directory \"%s\"\n", ldir);
+							FTPLogError(cip, kDoPerror, "Could not create local directory \"%s\"\n", ldir);
 							batchResult = kErrGeneric;
 							*cp = c;
 							continue;
@@ -228,7 +228,7 @@ FTPGetFiles3(
 			if (filePtr->type == 'l') {
 				(void) unlink(filePtr->lname);
 				if (symlink(filePtr->rlinkto, filePtr->lname) < 0) {
-					Error(cip, kDoPerror, "Could not symlink %s to %s\n", filePtr->rlinkto, filePtr->lname);
+					FTPLogError(cip, kDoPerror, "Could not symlink %s to %s\n", filePtr->rlinkto, filePtr->lname);
 					/* Note: not worth setting batchResult */
 				}
 			}
