@@ -10,7 +10,7 @@
 #	pragma hdrstop
 #endif
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	define ASCII_TRANSLATION 0
 #endif
 
@@ -32,7 +32,7 @@
 #endif
 
 int
-FTPListToMemory2(const FTPCIPtr cip, const char *const pattern, const LineListPtr llines, const char *const lsflags, const int blankLines, int *const tryMLSD)
+FTPListToMemory2(const FTPCIPtr cip, const char *const pattern, const FTPLineListPtr llines, const char *const lsflags, const int blankLines, int *const tryMLSD)
 {
 	char secondaryBuf[768];
 	char line[512];
@@ -117,7 +117,7 @@ FTPListToMemory2(const FTPCIPtr cip, const char *const pattern, const LineListPt
 			/* Not really fdopen, but close in what we're trying to do. */
 			result = kErrFdopenR;
 			cip->errNo = kErrFdopenR;
-			Error(cip, kDoPerror, "Could not fdopen.\n");
+			FTPLogError(cip, kDoPerror, "Could not fdopen.\n");
 			return (result);
 		}
 		
@@ -125,7 +125,7 @@ FTPListToMemory2(const FTPCIPtr cip, const char *const pattern, const LineListPt
 			result = SReadline(&lsSrl, line, sizeof(line) - 1);
 			if (result == kTimeoutErr) {
 				/* timeout */
-				Error(cip, kDontPerror, "Could not directory listing data -- timed out.\n");
+				FTPLogError(cip, kDontPerror, "Could not directory listing data -- timed out.\n");
 				cip->errNo = kErrDataTimedOut;
 				return (cip->errNo);
 			} else if (result == 0) {
@@ -134,7 +134,7 @@ FTPListToMemory2(const FTPCIPtr cip, const char *const pattern, const LineListPt
 				break;
 			} else if (result < 0) {
 				/* error */
-				Error(cip, kDoPerror, "Could not read directory listing data");
+				FTPLogError(cip, kDoPerror, "Could not read directory listing data");
 				result = kErrLISTFailed;
 				cip->errNo = kErrLISTFailed;
 				break;
@@ -234,7 +234,7 @@ FTPListToMemory2(const FTPCIPtr cip, const char *const pattern, const LineListPt
 
 
 int
-FTPListToMemory(const FTPCIPtr cip, const char *const pattern, const LineListPtr llines, const char *const lsflags)
+FTPListToMemory(const FTPCIPtr cip, const char *const pattern, const FTPLineListPtr llines, const char *const lsflags)
 {
 	return (FTPListToMemory2(cip, pattern, llines, lsflags, 1, (int *) 0));
 }	/* FTPListToMemory */
