@@ -1,6 +1,6 @@
 /* util.c
  *
- * Copyright (c) 1992-2001 by Mike Gleason.
+ * Copyright (c) 1992-2002 by Mike Gleason.
  * All rights reserved.
  * 
  */
@@ -24,7 +24,7 @@ int gNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = 0;
 static int gResolveSig;
 #endif
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #elif defined(HAVE_SIGSETJMP)
 sigjmp_buf gGetHostByNameJmp;
 #else	/* HAVE_SIGSETJMP */
@@ -200,171 +200,6 @@ MyInetAddr(char *dst, size_t siz, void *src, int i)
 
 
 
-
-/* This simplifies a pathname, by converting it to the
- * equivalent of "cd $dir ; dir=`pwd`".  In other words,
- * if $PWD==/usr/spool/uucp, and you had a path like
- * "$PWD/../tmp////./../xx/", it would be converted to
- * "/usr/spool/xx".
- */
-
-
-void
-CompressPath(char *const dst, const char *const src, const size_t dsize)
-{
-	int c;
-	const char *s;
-	char *d, *lim;
-	char *a, *b;
-
-#define isslash(c) ((c == '/') || (c == '\\'))
-	if (src[0] == '\0') {
-		*dst = '\0';
-		return;
-	}
-
-	s = src;
-	d = dst;
-	lim = d + dsize - 1;	/* leave room for nul byte. */
-	for (;;) {
-		c = *s;
-		if (c == '.') {
-			if (((s == src) || isslash(s[-1])) && (isslash(s[1]) || (s[1] == '\0'))) {
-				/* Don't copy "./" */
-				if (isslash(s[1]))
-					++s;
-				++s;
-			} else if (d < lim) {
-				*d++ = *s++;
-			} else {
-				++s;
-			}
-		} else if (isslash(c)) {
-			/* Don't copy multiple slashes. */
-			if (d < lim)
-				*d++ = *s++;
-			else
-				++s;
-			for (;;) {
-				c = *s;
-				if (isslash(c)) {
-					/* Don't copy multiple slashes. */
-					++s;
-				} else if (c == '.') {
-					c = s[1];
-					if (isslash(c)) {
-						/* Skip "./" */
-						s += 2;
-					} else if (c == '\0') {
-						/* Skip "./" */
-						s += 1;
-					} else {
-						break;
-					}
-				} else {
-					break;
-				}
-			}
-		} else if (c == '\0') {
-			/* Remove trailing slash. */
-			if (isslash(d[-1]) && (d > (dst + 1)))
-				d[-1] = '\0';
-			*d = '\0';
-			break;
-		} else if (d < lim) {
-			*d++ = *s++;
-		} else {
-			++s;
-		}
-	}
-	a = dst;
-
-	/* fprintf(stderr, "<%s>\n", dst); */
-	/* Go through and remove .. in the path when we know what the
-	 * parent directory is.  After we get done with this, the only
-	 * .. nodes in the path will be at the front.
-	 */
-	while (*a != '\0') {
-		b = a;
-		for (;;) {
-			/* Get the next node in the path. */
-			if (*a == '\0')
-				return;
-			if (isslash(*a)) {
-				++a;
-				break;
-			}
-			++a;
-		}
-		if ((b[0] == '.') && (b[1] == '.')) {
-			if (isslash(b[2])) {
-				/* We don't know what the parent of this
-				 * node would be.
-				 */
-				continue;
-			}
-		}
-		if ((a[0] == '.') && (a[1] == '.')) {
-			if (isslash(a[2])) {
-				/* Remove the .. node and the one before it. */
-				if ((b == dst) && (isslash(*dst)))
-					(void) memmove(b + 1, a + 3, strlen(a + 3) + 1);
-				else
-					(void) memmove(b, a + 3, strlen(a + 3) + 1);
-				a = dst;	/* Start over. */
-			} else if (a[2] == '\0') {
-				/* Remove a trailing .. like:  /aaa/bbb/.. */
-				if ((b <= dst + 1) && isslash(*dst))
-					dst[1] = '\0';
-				else
-					b[-1] = '\0';
-				a = dst;	/* Start over. */
-			} else {
-				/* continue processing this node.
-				 * It is probably some bogus path,
-				 * like ".../", "..foo/", etc.
-				 */
-			}
-		}
-	}
-#undef isslash
-}	/* CompressPath */
-
-
-
-void
-PathCat(char *const dst, const size_t dsize, const char *const cwd, const char *const src, int dosCompat)
-{
-	char *cp;
-	char tmp[512];
-
-	if (dosCompat != 0) {
-		if ((isalpha((int) cwd[0])) && (cwd[1] == ':')) {
-			/* A new fully-qualified DOS path was requested. */
-			CompressPath(dst, src, dsize);
-			return;
-		}
-	} else if (src[0] == '/') {
-		/* A new fully-qualified UNIX path was requested. */
-		CompressPath(dst, src, dsize);
-		return;
-	}
-	cp = Strnpcpy(tmp, cwd, sizeof(tmp) - 1);
-	if (dosCompat) {
-		if ((dst[1] != ':') || (dst[2] == '/'))
-			*cp++ = '/';
-		else
-			*cp++ = '\\';
-	} else {
-		*cp++ = '/';
-	}
-	*cp = '\0';
-	(void) Strnpcat(cp, src, sizeof(tmp) - (cp - tmp));
-	CompressPath(dst, tmp, dsize);
-}	/* PathCat */
-
-
-
 char *
 FileToURL(char *url, size_t urlsize, const char *const fn, const char *const rcwd, const char *const startdir, const char *const user, const char *const pass, const char *const hname, const unsigned int port)
 {
@@ -487,7 +322,7 @@ OurInstallationPath(char *const dst, const size_t siz, const char *const fname)
 void
 InitOurDirectory(void)
 {
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	DWORD dwType, dwSize;
 	HKEY hkey;
 	char *cp;
@@ -613,7 +448,7 @@ InitOurDirectory(void)
 void
 InitUserInfo(void)
 {
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	DWORD nSize;
 	char *cp;
 
@@ -783,7 +618,7 @@ AbsoluteToRelative(char *const dst, const size_t dsize, const char *const dir, c
 
 
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #else
 
 /* Some commands may want to jump back to the start too. */
@@ -808,7 +643,7 @@ CancelGetHostByName(int sigNum)
 int
 MyGetHostByName(char *const volatile dst, size_t dsize, const char *const hn, int t)
 {
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	struct hostent *hp;
 	struct in_addr ina;
 
@@ -1024,7 +859,6 @@ strncoll(const char *a, const char *b, size_t n)
 		(void) strncpy(sa, a, n);
 		(void) strncpy(sb, b, n);
 		rc = (strcoll(sa, sb));
-		return (rc);
 	} else {
 		char *ma, *mb;
 
@@ -1042,8 +876,8 @@ strncoll(const char *a, const char *b, size_t n)
 		rc = (strcoll(ma, mb));
 		free(ma);
 		free(mb);
-		return (rc);
 	}
+	return (rc);
 }	/* strncoll */
 #endif
 
@@ -1054,7 +888,7 @@ int
 DecodeDirectoryURL(
 	const FTPCIPtr cip,	/* area pointed to may be modified */
 	char *url,		/* always modified */
-	LineListPtr cdlist,	/* always modified */
+	FTPLineListPtr cdlist,	/* always modified */
 	char *fn,		/* always modified */
 	size_t fnsize
 )
@@ -1085,7 +919,7 @@ DecodeDirectoryURL(
 
 
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 void SysPerror(const char *const errMsg)
 {
 	char reason[128];
