@@ -13,6 +13,11 @@ typedef volatile sigproc_t vsigproc_t;
 #define STREQ(a,b) (strcmp(a,b) == 0)
 #define STRNEQ(a,b,s) (strncmp(a,b,(size_t)(s)) == 0)
 
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+#	define ISTRCMP stricmp
+#	define ISTRNCMP strnicmp
+#endif
+
 #ifndef ISTRCMP
 #	ifdef HAVE_STRCASECMP
 #		define ISTRCMP strcasecmp
@@ -46,7 +51,7 @@ typedef volatile sigproc_t vsigproc_t;
 #define kPasswordMagic "*encoded*"
 #define kPasswordMagicLen 9
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	define kOurDirectoryName	"ncftp"
 #	define kNcFTPBookmarksMailslot "\\\\.\\mailslot\\ncftpbm.slt"
 #	define kNcFTPBookmarksMailslotMsgSize 128
@@ -96,9 +101,9 @@ int StrToBool(const char *const);
 void AbsoluteToRelative(char *const, const size_t, const char *const, const char *const, const size_t);
 int MyGetHostByName(char *const volatile, size_t, const char *const, int);
 time_t UnDate(char *dstr);
-int DecodeDirectoryURL(const FTPCIPtr, char *, LineListPtr, char *, size_t);
+int DecodeDirectoryURL(const FTPCIPtr, char *, FTPLineListPtr, char *, size_t);
 char *OurInstallationPath(char *const dst, const size_t siz, const char *const fname);
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 void SysPerror(const char *const errMsg);
 #endif
 
